/* Programmer: Steven Farago
 * Class:      EEL 5666
 * Description: This file is the primary location for the behaviors
                that control the Capture the Flag robots
 */

#include <serial.h>
#include <string.h>
#include <mil.h>
#include <hc11.h>
#include "defs.h"


void be_disabled();

/* IR code notes/requirements
 *** Must begin with at least one 1 (a series of 1's preferable)
        Detection of the code starts at the first 0 after this
        series of 1's (--> The actual received code cannot start with 1)
*** 4 successive 0s terminate the code.
*/

#define REV_CODE_SIZE 40
#define DISABLE_CODE_SIZE 16
#define COMM_PERIOD 2000

/* Define globals */
int send_code[20];
int send_code_size;
int send_index;
int send_complete;

char *my_disable_code="1101011010010101";
char *moyer_code="0011110000";

int rev_code[REV_CODE_SIZE];
int rev_code_index;
int code_received;
int successive_zeros;
int rev_code_ip;

#define TOC5_ISR send_code_isr
#define TOC4_ISR detect_code_isr
#pragma interrupt_handler send_code_isr
#pragma interrupt_handler detect_code_isr

int SIGNAL();

void send_code_isr(void)
{
 TOC5=(TOC5+COMM_PERIOD);  /* Setup OC5 period for baud rate defined above */
 CLEAR_FLAG(TFLG1, 0x08);
 /* Send code until end of code array is reached */
 if (send_index<send_code_size)
    {if (send_code[send_index++] == 0)   {SET_BIT(TCTL1, 0x02);
                                          CLEAR_BIT(TCTL1, 0x01);
                                         }
     else                                {SET_BIT(TCTL1, 0x03);
                                          }                                       
    }
 else
    {SET_BIT(TCTL1, 0x02);
     CLEAR_BIT(TCTL1, 0x01);
     send_index=0;
    }
return;
}


void detect_code_isr()
{int sig;

 TOC4=(TOC4+COMM_PERIOD);
 CLEAR_FLAG(TFLG1, 0x10);

 sig=SIGNAL();
/* printf("sig is %d \n",sig);*/

/* If currently in the middle of receiving a code, wait for a 1 to start
   receiving a new code.  If we get a 1, wait for a 0, and then "sync
   up" in the middle of the 0.
*/

 if (! rev_code_ip)
   {if (sig==1)      {rev_code_ip=1; rev_code_index=0;
                      while (sig=SIGNAL()==1);
                      TOC4=(TCNT+COMM_PERIOD+(COMM_PERIOD/2));
                      rev_code[++rev_code_index]=sig;
                     }
     else {return;}
   }
 else
   {rev_code[++rev_code_index]=sig;
    if (rev_code_index>=REV_CODE_SIZE) {rev_code_ip=0;
                                        code_received=1;
                                        CLEAR_BIT(TMSK1, 0x10);
                                       }

   }
}

void init_send_system()
/* Function:    Initialize OC5 for use with IR code sending system
   Inputs:      None
   Outputs:     None
   Notes:       You must call this function prior to using IR code sending
                system
*/

{
 INTR_OFF();

 /* Initialize send system control global vars */
 send_complete=send_code_size=send_index=0;

 CLEAR_BIT(PACTL, 0x04);     /* Set OC5/I4 pin for OC5 use */
 CLEAR_BIT(TMSK1, 0x08);     /* Initially disable OC5 interrupts */
  
 SET_BIT(TCTL1, 0x02);       /* Set OC5 to output low initially */
 CLEAR_BIT(TCTL1, 0x01);

 CLEAR_BIT(TCTL2,0x03);      /* Setup IC3 pin for input            */
 INTR_ON();
}

 
void init_rev_system()
{
 INTR_OFF();

 rev_code_index=rev_code_ip=code_received=successive_zeros=0;
/* CLEAR_BIT(TCTL1, 0x0C) */         /* Use OC4 only as timer.  */
 SET_BIT(TMSK1, 0x10);          /* Enable OC4 interrupts */


 INTR_ON();
}

/* Weirdo behavior !!! */
int check_disable()
{unsigned int i, msg, my_code;
 msg=my_code=0;
 if (code_received)
    {
     code_received=0;
     for (i=0; i<DISABLE_CODE_SIZE; ++i)
        {my_code=my_code << 1;
         if (my_disable_code[i]=='1') {++my_code;}
        }
     for (i=0; i<16; ++i)
        {msg=msg << 1;
         if (rev_code[i]==1) {++msg;}
         
        }
     for (i=16; i<40; ++i)
        {
         if (msg==my_code) {return(1);}
         msg=msg << 1;
         if (rev_code[i]==1) {++msg;}
        }
   }

SET_BIT(TMSK1, 0x10);
return(0);
}

void send_message(char *message)
{int i;
 for (i=0; i<=strlen(message); ++i)
   {if (message[i]=='0') {send_code[i]=0;}
    else {send_code[i]=1;}
   }
 send_index=send_complete=0;
 send_code_size=strlen(message);
 SET_BIT(TMSK1, 0x08);
 return;
}


void send_off()
{CLEAR_BIT(TMSK1, 0x08);}


int SIGNAL()
 {
 if ((PORTA & 0x01) == 1) {return(0);} else {return(1);}
}

void wait(int seconds)
{int i,j,k;
 i=j=10000*seconds;
 while(i--);
 while(j--);
 while(k--);
}

/*
void main()
{char *message="1101101011010101110111101101110101110010";




 int i,j,idx,done;


 init_serial();
 init_timers();
 
 init_rev_system();
 done=0;

 printf("BEGIN!!!!! \n\n");
 while(! done)
 {if (rev_code_index>=REV_CODE_SIZE) {done=1;}}

    for (i=0; i<40; ++i)
     {printf("%d",rev_code[i]);
     }
   printf("\n");
  CLEAR_BIT(TMSK1, 0x10);
  printf("Above is rev code \n");
  check_disable();
 */
 /*
 for (j=0; j<400; ++j)
 {send_code[j]=rev_code[j];}
 send_index=send_complete=0;
 send_code_size=400;

 SET_BIT(TMSK1, 0x08);

 printf("Sending message\n");
 while(1);
*/

/*
printf("Begin \n");
for(i=0; i<40; ++i)
{if (message[i]=='1') {rev_code[i]=1;}
 else {rev_code[i]=0;}
}
 check_disable();
*/

