#include <mil.h>
#include <hc11.h>
#include <serial.h>
#include <motor.h>
#include "songs.c"

int note[97];
int style=0;
void pause(int t);


/* Definitions for the style variable.  Setting the style variable to one
   of these will cause music to be played in that style
*/
#define NORMAL 0
#define STACATTO 1
#define LEGATO 2


#define TOC1_ISR music_isr
#pragma interrupt_handler music_isr

unsigned int toggle;
unsigned int MUS_PERIOD;
unsigned int wait_rate;

/* Drives OC1 to produce the correct frequency for the given note.
   the correct period is specified by MUS_PERIOD variable
*/

void music_isr()
{
 CLEAR_FLAG(TFLG1, 0x80);
 TOC1=TOC1+MUS_PERIOD;
 if (OC1D && 0x80 && toggle) {CLEAR_BIT(OC1D, 0x80);}
 else {SET_BIT(OC1D, 0x80);}

}

/* Defines the correct periods associated with the notes.  Note 1
   corresponds to some low C.  Each index increment represents a half
   step up in the chromatic scale (i.e. note 2 = C#, note 3 = D etc...
   A future modification will involve creating a mapping between these
   numbers and the traditional characters associated with the music
   (i.e.. C, D ,E , F etc...)
 */
void init_music()
 {
  note[1]=28902; note[2]=27248; note[3]=25707; note[4]=24272; 
  note[5]=22883; note[6]=21645; note[7]=20408; note[8]=19268; 
  note[9]=18182; note[10]=17153; note[11]=16207; note[12]=15291; 
  note[13]=14430; note[14]=13624; note[15]=12853; note[16]=12136; 
  note[17]=11455; note[18]=10811; note[19]=10204; note[20]=9634; 
  note[21]=9091; note[22]=8584; note[23]=8097; note[24]=7645; 
  note[25]=7215; note[26]=6812; note[27]=6427; note[28]=6068; 
  note[29]=5727; note[30]=5405; note[31]=5102; note[32]=4815; 
  note[33]=4545; note[34]=4290; note[35]=4050; note[36]=3823; 
  note[37]=3608; note[38]=3405; note[39]=3214; note[40]=3034; 
  note[41]=2864; note[42]=2703; note[43]=2551; note[44]=2408; 
  note[45]=2273; note[46]=2145; note[47]=2025; note[48]=1911; 
  note[49]=1804; note[50]=1703; note[51]=1607; note[52]=1517; 
  note[53]=1432; note[54]=1351; note[55]=1276; note[56]=1204; 
  note[57]=1136; note[58]=1073; note[59]=1012; note[60]=956; 
  note[61]=902; note[62]=851; note[63]=804; note[64]=758; 
  note[65]=716; note[66]=676; note[67]=638; note[68]=602; 
  note[69]=568; note[70]=536; note[71]=506; note[72]=478; 
  note[73]=451; note[74]=426; note[75]=402; note[76]=379; 
  note[77]=358; note[78]=338; note[79]=319; note[80]=301; 
  note[81]=284; note[82]=268; note[83]=253; note[84]=239; 
  
  init_songs();
  CLEAR_BIT(OC1M, 0xff);
  SET_BIT(OC1M, 0x80);
 
  SET_BIT(PACTL, 0x80);
  
  SET_BIT(TMSK1, 0x80);
  toggle=1;
}


void sound_on()
{toggle=1;}

void sound_off()
{toggle=0;}

/* Plays given note until a sound_off() call is issued */
void play_note(int note_num)
{
 MUS_PERIOD=note[note_num];
 sound_on();
}

/* Format for play funtion 
 * Takes int as input 
 * MSB 8 bits of int are duration (0-15 0=16th note, 15=whole note)
 * LSB 8 bits of int are note (1-84) 1=low C  
 */

void play(int muscmd)
 {
  int notenum, duration;
/*    printf("Muscmd is %d \n",muscmd);  */
    notenum=muscmd & 0x00ff;
    duration=muscmd & 0xff00;
    sound_on();
    if (notenum==0) {sound_off();}
    else {/* printf("Note: %d ... Duration %d \n",notenum,duration); */
          MUS_PERIOD=note[notenum]; sound_on();}
    pause(duration);
    return;
 }

void beep()
{play(558);}

/* Carefully timed pause function.  The int t corresponds to a duration
 * value.  Each increment of t represents a 16th note long duration.
 * So, t=4 corresponds to a quarter note pause.  t=8 to a half note pause.
 * The pause length is dependent on the wait_rate global var.  This var is
 * usually computed the composed perl script so that tempo quarter notes
 * (<tempo> defined in music file) occur per minute.
 */
void pause(int t)
{
 int target;
 int current;
 unsigned int on_time,off_time,time,i,j,k;

 sound_on();

 time=t/0x100;
 switch (style)    /* 1= staccto 2=legato else normal   */
   {case 1 : if (time>1) {on_time=2; off_time=time-on_time;}
             else {on_time=1; off_time=0;} break;
    case 2 : on_time=time; off_time=0; break;
    default: if (time<2) {on_time=1; off_time=0;}
             else {on_time=time-1; off_time=1;}
   }

 for(i=1; i<=on_time; ++i)
  {j=wait_rate;
   while(j--) {k=20; while(k--);}
  }

 sound_off();    
 for(i=1; i<=off_time; ++i)
  {j=wait_rate;
   while(j--) {k=20; while(k--);}
  }
sound_off();
}


void play_song(unsigned int *song)
{int i=1;
 wait_rate=song[0];
 printf("Wait rate is %d \n",wait_rate);
 while((song[i] & 0x00ff) != 0xff)
    {play(song[i++]);}
 sound_off();
}

/* Fairly decent sound of a gun */
void gun_sound(int time)
{unsigned int k,j,i;
 while(time--)
 {
  for (j=0; j<=14; ++j)
   {
    for(i=84; i>=20; --i)
      {play_note(i);
       k=75;
       while(k--);
      }
    }
 }
 sound_off();
}

/*
void main()
{int i,j;
 init_serial();
 
 init_music();
 init_songs();
 printf("GO!!!! \n");
 INTR_ON();

 play_song(spy);

}


*/
