# Programmer: Steven Farago
# Class: EEL 5666
# Description:  This perl script allows easy conversion of a legible
#       music composition for use in the music.c file.
#
# Note: type "perl songs.pl" at the DOS prompt to begin this script
# (assuming perl is in current directory or in path).
#
# Copies of perl can be obtained from the perl archive at www.cise.ufl.edu.
#
# The script accepts a file as input works as follows:
# The first line of the input file should contain the name of the script.
# The next line should contain the tempo (in quarter notes/minute).
# Following lines contain music commands.  The number of commands per line
# is unimportant, so long as the line is not blank (this terminates the
# script).
#
# A music command consists of three parts separated by colons.  First is
# the duration of the note.  This is a number from 1 to 16 where each
# addition of a 1 represents a 16th count.  So 4 represents a quarter note.
#
# Next is the note you wish played.  See the valid_notes list below for a
# list of valid notes (duh.)
#
# Finally is the octave of the note.  Valid octaves are 0 to 6 (though the
# speaker produces poor sounds at the extreme octaves).
#
#
# So a command 4:C:4 would represent a 4th octave C quarter note.  See the
# .sng files for more comprehensive examples.  Note: All music commands must
# be separated by at least 1 space.
#
# Complete code for use with the music.c/songs.c files is created in the
# outfile.out file upon successful termination of the script.  You have
# only to copy the code to the songs.c file, enter the new song's name in
# the init file in songs.c, and issue a play_song(song) command from
# music.c to have the HC11 play a song. 


$valid_notes[0]='C' ; $valid_notes[1]='C#' ; $valid_notes[2]='Cb';
$valid_notes[3]='D' ; $valid_notes[4]='D#' ; $valid_notes[5]='Db';
$valid_notes[6]='E' ; $valid_notes[7]='E#' ; $valid_notes[8]='Eb';
$valid_notes[9]='F' ; $valid_notes[10]='F#'; $valid_notes[11]='Fb';
$valid_notes[12]='G'; $valid_notes[13]='G#'; $valid_notes[14]='Gb';
$valid_notes[15]='A'; $valid_notes[16]='A#'; $valid_notes[17]='Ab';
$valid_notes[18]='B'; $valid_notes[19]='B#'; $valid_notes[20]='Bb';
$valid_notes[21]='R'; $valid_notes[22]='X' ;

$note_num[0]=1  ; $note_num[1]=2  ;  $note_num[2]=12;
$note_num[3]=3  ; $note_num[4]=4  ;  $note_num[5]=2 ;
$note_num[6]=5  ; $note_num[7]=6  ;  $note_num[8]=4 ;
$note_num[9]=6  ; $note_num[10]=7 ;  $note_num[11]=5 ;
$note_num[12]=8 ; $note_num[13]=9 ;  $note_num[14]=7 ;
$note_num[15]=10; $note_num[16]=11;  $note_num[17]=9 ;
$note_num[18]=12; $note_num[19]=1 ;  $note_num[20]=11;
$note_num[21]=0 ; $note_num[22]=0xff ;

printf("Name of Source File: ");
$source=<STDIN>; chop($source);
open(NEWSONG,"<$source");
$got_tempo=0; 
$counter=1;
$song_name=<NEWSONG>; chop($song_name); printf("song $song_name \n");
$tempo=<NEWSONG>; chop($tempo); printf("tempo $tempo \n");

$wait_rate=int((60/$tempo)/.000956); 

$song_name[0]=$wait_rate;

while ($newsong=<NEWSONG>)
   {
    chop($newsong);
    printf("newsong is $newsong \n");
    $i=0;
    @cmd=split(' ',$newsong);
    while($next=$cmd[$i++])
    {($length, $note, $octave) = split(':',$next);
       $j=0; $number=99;
       while($j<23)
          {if ($note eq $valid_notes[$j]) {$number=$j;}
           $j++;
          }

       if ($number == 99) {die("Invalid Note at position $counter");}

       if (($length>15) || ($length<0))
         {die("Length out of range at position $counter");}

       if (($octave>6) || ($octave<0))
         {die("Octave out of range at position $counter");}


       $ret=(256*$length)+($note_num[$number]+($octave*12));
       $song_name[$counter]=$ret;
       $counter++; ;
      }
   }
open(OUT,">outfile.out");
$song_name[$counter]=255;
$size=$counter+1;
print OUT ("unsigned int $song_name"."[$size] ; \n");
print OUT ("void $song_name"."_sng()  \n");
print OUT (" { \n");
print OUT ("   $song_name"."[0] = $wait_rate ; \n   ");
$i=1;
while($song_name[$i])
{print OUT ("$song_name"."[$i] = $song_name[$i] ;  ");
 if ($i%3 == 0) {print OUT ("\n   ");}
 $i++;}
print OUT ("\n } \n");
close OUT;
