#include <hc11.h>
#include <mil.h>
#include <serial.h>
#include <int32.h>

#define NUMBER_OF_TIMERS 11
/* 32 bit timer includes a max running time of about 35 minutes (before  */
/* timer system overflows)                                               */

/* Timers not associated with motor controlling behaviors */

#define CONTROLLER_TIMER 8
#define SHOOT 9
#define ONLY_ONE_TURN 10



INT32 time;
INT32 timer[NUMBER_OF_TIMERS];
unsigned int timehi=0;

#define TOF_ISR timer_isr
#pragma interrupt_handler timer_isr

void timer_isr()
{
 CLEAR_FLAG(TFLG2, 0x80);
 ++timehi;
}
                                                                  
/* Compares two 32 bit integers                                 */
/* Returns 1 if left>right, -1 if left<right, 0 if left=right   */
/* ------------------------------------------------------------ */
int int32_comp(INT32 left, INT32 right)
{
 unsigned int leftlo, lefthi, rightlo, righthi;
 leftlo=left.lo_word; lefthi=left.hi_word;
 rightlo=right.lo_word; righthi=right.hi_word;

  if (lefthi>righthi) {return(1);}
  else {if (lefthi<righthi) {return(-1);}
        else {if (leftlo>rightlo) {return(1);}
              else {if (leftlo<rightlo) {return(-1);}
                    else {return(0);}
                   }
             }
       }
}

/* Initializes timer system  */
/* ------------------------  */

void init_timers()
{int i;
 timehi=time.lo_word=time.hi_word=0;

 for (i=0; i<NUMBER_OF_TIMERS; ++i)
   {timer[i]=int_to_int32(0);}
       
 SET_BIT(TMSK2, 0x80);
}
  
/* Updates current global system time variable */

INT32 update_time()
{
 time.hi_word=timehi;
 time.lo_word=TCNT;
 return(time);
}

/* Set timer <number> for <delay> microseconds */
/* Max delay is 36767 microseconds */

int set_timer(unsigned int number, unsigned int delay)
{INT32 expires;
 
 update_time();
 expires=int32_add_int(time, delay);
 expires=int32_add_int(expires, delay);


 if (number>=NUMBER_OF_TIMERS) {return(-1);}
    else {
          if (int32_comp(timer[number], time) == -1)
             {timer[number]=expires; 
              return(0);
             }
         }
 return(1);
}

/* Inputs are timer number along with high and low word of delay time   */
/* in hex                                                               */

int set_timer_long(unsigned int number, unsigned int delay_high, unsigned int delay_low)
{INT32 expires, delay;

 update_time();
 delay.hi_word=delay_high; delay.lo_word=delay_low;
 expires=int32_add(time, delay);

 if (number>=NUMBER_OF_TIMERS) {return(-1);}
 else {if (int32_comp(timer[number], time) == -1)
             {timer[number]=expires; 
              return(0);
             }
      }
 return(1);
}

/* Checks timer <number>.  Returns 1 if expired, 0 otherwise */
int check_timer(unsigned int number)
{INT32 expire;

 if (number>NUMBER_OF_TIMERS) {return(-1);}
 else {expire=timer[number];}

 if (int32_comp(update_time(), expire) == 1) {return(1);}
 else {return(0);}
}




void p_int32(INT32 in)
{printf("hi %u lo %u \n",in.hi_word, in.lo_word);}

/*
void main()
{

 INT32 a,b;

 init_timers();
 init_serial();
 INTR_ON();
 a.hi_word=0x2152; a.lo_word=0x5431;
 b.hi_word=0x1254; b.lo_word=0x4345;

 printf("Trial a>b....\n");
 printf("a   "); p_int32(a);
 printf("b   "); p_int32(b);
 printf("compare result %d \n\n",int32_comp(a,b));

 a.hi_word=0x6522; a.lo_word=0x5431;
 b.hi_word=0x9942; b.lo_word=0x4345;

 printf("Trial a<b....\n");
 printf("a   "); p_int32(a);
 printf("b   "); p_int32(b);
 printf("compare result %d \n\n",int32_comp(a,b));


 a.hi_word=0x4234; a.lo_word=0xabcd;
 b.hi_word=0x4234; b.lo_word=0xabcd;

 printf("Trial a=b....\n");
 printf("a   "); p_int32(a);
 printf("b   "); p_int32(b);
 printf("compare result %d \n\n",int32_comp(a,b));
printf("begin... \n");
set_timer(3, 0x2674);
b=time;
while(! check_timer(3));
a=update_time();
printf("before ... "); p_int32(b);

printf("after ...  "); p_int32(a);

} 

*/
